
/*===============================================*\
  
  Mercredi 13 Decembre 2000  

  Arash HABIBI
  
  SocketToolbox.c
  
  Un ensemble de fonctions qu'on utilise
  partout en manipulant les sockets.  
  
\*================================================*/


#include "SocketToolbox.h"

/*------------------------------------------------*\
  Fonction qui, a partir du nom d'un hote, 
  fournit l'adresse IP (long sous format hote)
\*------------------------------------------------*/

static u_long _IPAddrByName(char *Nom)
{
  u_long AdresseIP; 
  struct hostent *HostEnt; 
  
  HostEnt = gethostbyname(Nom); 
  if(HostEnt==0) 
    {
      fprintf(stderr,"_IPAddrByName : Pas d'hote du nom de %s\n",Nom); 
      exit(1);
    }
  
  memcpy(&AdresseIP, HostEnt->h_addr, HostEnt->h_length); 
  
  return(AdresseIP);
}



/*------------------------------------------------*\
  Fonction qui a partir d'un entier long
  (consideree comme une adresse IP au format
  hote) fournit les quatre entiers tels que 
  l'adresse IP est representee sour la forme
  habituelle de n1.n2.n3.n4
\*------------------------------------------------*/

static void _LongToAdresseIP(u_long A_IP, AdresseIP_t *habituel)
{
  int reste;
  
  habituel->n1 = A_IP / 0x1000000; 
  reste = A_IP % 0x1000000; 
  
  habituel->n2 = reste / 0x10000; 
  reste = reste % 0x10000; 
	
  habituel->n3 = reste / 0x100; 
  habituel->n4 = reste % 0x100; 
}



/*------------------------------------------------*\
  A partir de donnees de l'hote prepare une
  adresse format reseau prete a etre envoyee. 
\*------------------------------------------------*/

struct sockaddr_in ST_WriteNetworkAddress(char *HostName, u_short port)
{
  struct sockaddr_in adresse_dist; 
  
  adresse_dist.sin_family      = AF_INET;
  adresse_dist.sin_port        = htons(port);  
  adresse_dist.sin_addr.s_addr = _IPAddrByName(HostName); 
  
  /* Ici il n'y a plus besoin de htonl car la fonction */
  /* gethostbyname donne des adresses IP en format reseaux */
	
  return(adresse_dist);
}



/*------------------------------------------------*\
  A partir d'une adresse format reseau, produit
  l'adresse format hote sous la forme habituelle
  de n1.n2.n3.n4
\*------------------------------------------------*/

void ST_ReadNetworkAddress(struct sockaddr_in adresse_dist, AdresseIP_t *IP, u_short *port)
{
  *port = ntohs(adresse_dist.sin_port);
  _LongToAdresseIP(ntohl(adresse_dist.sin_addr.s_addr), IP);			  
}



/*------------------------------------------------*\
  A partir d'un type de socket (SOCK_DGRAM ou 
  SOCK_STREAM) d'un numero de port, fournit
  une adresse (au format hote) de socket 
  locale prete a etre utilisee. 
\*------------------------------------------------*/

int ST_GetAndBindSocket(int type, short nb_port, struct sockaddr_in *adresse_loc)
{
	int desc, res, taille; 

	if(adresse_loc==NULL) return(-1); 
	
/* Il faut que adresse_loc soit deja allouee */

	desc = socket(AF_INET, type, 0);

	if(desc == -1) {perror("ST_GetAndBindSocket (appel a socket)"); exit(1);}
	
	adresse_loc->sin_family      = AF_INET;
	adresse_loc->sin_port 	     = htons(nb_port);          
	adresse_loc->sin_addr.s_addr = htonl(INADDR_ANY);	
	
/*---- Attachement effective de la socket locale ---*/

	res = bind(desc, (struct sockaddr *)adresse_loc, 
		sizeof(*adresse_loc));
	
	if(res == -1) {perror("ST_GetAndBindSocket (appel a bind)"); exit(1);}
	
/*---- Recupere l'adresse effective de l'attachement ---*/
/*---- (pour le cas ou le numero de port n'a pas ete ---*/
/*---- explicitement specifie (0) ----------------------*/
/*---- getsockname donne une adresse au format reseau --*/
	
	taille = sizeof(*adresse_loc);
	
	res = getsockname(desc, (struct sockaddr *)adresse_loc, 
		&taille);
	
/* -- Pour avoir l'adresse sous format hote ----------- */
	
	adresse_loc->sin_port = ntohs(adresse_loc->sin_port);
	adresse_loc->sin_addr.s_addr = 
	       ntohl(adresse_loc->sin_addr.s_addr);

	return(desc);
}


/*------------------------------------------------*\
  Retourne 1 si les deux adresses sont identiques
  et 0 sinon. 
\*------------------------------------------------*/

extern int ST_SameAddress(struct sockaddr_in adr_1, struct sockaddr_in adr_2)
{
  if((adr_1.sin_family      == adr_2.sin_family)&&
     (adr_1.sin_port        == adr_2.sin_port)&&
     (adr_1.sin_addr.s_addr == adr_2.sin_addr.s_addr))
    return(1); 
  else 
    return(0); 
  
}


/*------------------------------------------------*\
  Affiche  l'cran l'adresse IP et numro de port 
  correspondant  adresse. 
\*------------------------------------------------*/

extern void ST_CheckNetworkAddress(struct sockaddr_in adresse, char *comment)
{
  struct AdresseIP_t AIP; 
  u_short port; 

  ST_ReadNetworkAddress(adresse, &AIP, &port); 
  fprintf(stderr,"%s : %d.%d.%d.%d : %d\n", comment, AIP.n1, AIP.n2, AIP.n3, AIP.n4, port); 

}
